@@ -43,20 +43,20 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
       x <- x[, !na.check]
     }
 
-    # if requested: check and remove constant series
+    # if requested: check for constant series and remove
     if (drop.const.series) {
-      # old: cst.check <- sapply(x, function(x) var(as.numeric(x), na.rm = TRUE)==0) # old
+      # old: cst.check <- sapply(x, function(x) var(as.numeric(x), na.rm = TRUE)==0)
       # -> var() and sd() on factors is deprecated as of R 3.2.3 -> use duplicated()
       cst.check <- sapply(x, function(x) {
                               if (is.factor(x) || is.character(x)) {
                                 all(duplicated(x[!is.na(x)])[-1L])
                               } else {
-                                x[!is.finite(x)] <- NA # set infinite elements to NA only for check
-                                var(as.numeric(x), na.rm = TRUE)==0
+                                x[!is.finite(x)] <- NA # infinite elements set to NA only for this check
+                                var(as.numeric(x), na.rm = TRUE) == 0
                               }
                             })
       
-      # following line: bug fixed thanks to Marciej Szelfer 
+      # following line: bug fixed thanks to Marciej Szelfer
       cst.check <- cst.check | is.na(cst.check)
       cst.serie <- names(x)[cst.check]
       if (length(cst.serie) > 0){
@@ -95,12 +95,12 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         time <- NULL
     }
     # if the length of index is 1, id = index and time is NULL
-    if (length(index)==1){
+    if (length(index) == 1){
         id <- index
         time <- NULL
     }
     # if the length of index is 2, the first element is id, the second is time
-    if (length(index)==2){
+    if (length(index) == 2){
         id <- index[1]
         time <- index[2]
     }
@@ -119,15 +119,15 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     if(is.numeric(id.name)){
         if(!is.null(time.name)){warning("The time index (second element of 'index' argument) will be ignored\n")}
         N <- nrow(x)
-        if( (N %% id.name)!=0){
+        if( (N %% id.name) != 0){
             stop(paste0("unbalanced panel, in this case the individual index may not be indicated by an integer\n",
                         "but by specifying a column of the data.frame in the first element of the 'index' argument\n"))
         }
         else{
             T <- N %/% id.name
             n <- N %/% T
-            time <- rep(1:T, n)
-            id <- rep(seq(1:n), rep(T, n))
+            time <- rep((1:T), n)
+            id <- rep((1:n), rep(T, n))
             id.name <- "id"
             time.name <- "time"
             if (id.name %in% names(x)) warning(paste0("column '", id.name, "' overwritten by id index"))
@@ -157,7 +157,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
             n <- length(Ti)
             time <- c()
             for (i in 1:n){
-                time <- c(time,1:Ti[i])
+                time <- c(time, 1:Ti[i])
             }
             time.name <- "time"
             if (time.name %in% names(x)) warning(paste0("column '", time.name, "' overwritten by time index"))
@@ -205,7 +205,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     posindex <- match(c(id.name, time.name, group.name), names(x))
     index <- x[, posindex]
     if (drop.index) {
-        x <- x[ , - posindex]
+        x <- x[ , -posindex, drop = FALSE]
         if (ncol(x) == 0L) cat("after dropping of index variables, the pdata.frame contains 0 columns")
     }
     
@@ -218,9 +218,9 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     if (row.names) {
         attr(x, "row.names") <- fancy.row.names(index)
         # NB: attr(x, "row.names") allows for duplicate rownames (as
-        # opposed to row.names(x) <- something) no fancy row.names for
-        # index attribute (!?): maybe because so it is possible to
-        # restore original row.names?
+        # opposed to row.names(x) <- something)
+        # NB: no fancy row.names for index attribute (!?):
+        # maybe because so it is possible to restore original row.names?
     }
     
     class(index) <- c("pindex", "data.frame")
@@ -234,56 +234,55 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
   if (inherits(value, "pseries")){
     # remove pseries features before adding value as a column to pdata.frame
     if (length(class(value)) == 1) value <- unclass(value)
-    else {
-      value <- check_propagation_correct_class(value) # check if propagation to a higher type happened prior to assignment, e.g. integer => numeric
-      attr(value, "class") <- setdiff(class(value), "pseries") # don't use class(value) <- here as it can force the storage mode to change
-    }
+    else attr(value, "class") <- setdiff(class(value), "pseries")
     attr(value, "index") <- NULL
   }
   "$<-.data.frame"(x, name, value)
 }
 
-# NB: We don't have methods for [<-.pdata.frame and [[<-.pdata.frame, so it
-#     dispatches to the respective data.frame methods.
-#     This results in really assigning a pseries to the pdata.frame in case 
-#     of [<- and [[<- as can be seen by lapply(some_pdata.frame, class) after 
+# NB: We don't have methods for [<-.pdata.frame and [[<-.pdata.frame, so these functions
+#     dispatch to the respective data.frame methods which assign whatever is
+#     handed over to the methods. Especially, if a pseries is handed over, this
+#     results in really assigning a pseries to the pdata.frame in case of usage of
+#     [<- and [[<-. This is inconsistent because the columns of a pdata.frame do not
+#     have the 'pseries' features.
+#     This can be seen by lapply(some_pdata.frame, class) after 
 #     assigning with the respective .data.frame methods
 
 
-# NB: currently no extracting/subsetting function for class pseries, thus
+# NB: Currently no extracting/subsetting function for class pseries, thus
 #     vector subsetting is used which removes the pseries features
-#    There is a working sketch below, but check if it does not interfere with anything else
+#
+#     Test cases in tests/test_pdata.frame_subsetting.R 
+# 
+#     There is a working sketch below, but check if it does not interfere with anything else.
+#     * check if it works with FD and between models (these models compress the data and, thus,
+#       pmodel.reponse for these does not return a pseries but a pure numeric)
+#
 # "[.pseries" <- function(x, ...) {
 # 
-#   ## use '...' instead of only one specific argument, because subsetting for
-#   ## factors can have argument 'drop', e.g., x[i, drop=TRUE] see ?Extract.factor
-# 
+#  ## use '...' instead of only one specific argument, because subsetting for
+#  ## factors can have argument 'drop', e.g., x[i, drop=TRUE] see ?Extract.factor
+# #stop()
 #   index <- attr(x, "index")
-# 
-#   # to identify the entries which we need to keep in the index:
-#   #  use names of the vector, but set names to integer sequence first (safer)
-#   #  -> use this information (names_subsetted) to subset the index
-#   #  this way, we can use the regular vector subsetting of R x[i] without
-#   #  worrying about the form of i (logical, numeric, character, some expression, ...)
+#   if (is.null(index)) warning("pseries object with is.null(index(pseries)) == TRUE encountered, trying to continue anyway...")
+#   if (!is.index(index)) stop(paste0("pseries object has illegal index with class(index) == ", paste0(class(index), collapse = ", ")))
 #   names_orig <- names(x)
-#   names(x) <- seq_along(x)
-#   # remove class 'pseries' and index attrib to use R's vector subsetting x[i]
-#   class(x) <- setdiff(class(x), "pseries")
-#   attr(x, "index") <- NULL
+#   x <- remove_pseries_features(x)
 #   result <- x[...]
-#   names_subsetted <- as.numeric(names(result))
 # 
-#   # make result a 'pseries' again:
-#   # add back to result:
-#   #    * subsetted original names                # TODO: not needed?
-#   #    * subsetted index as attribute
-#   #    * class 'pseries'
-# #  names(result) <- names_orig[names_subsetted]           # TODO: not needed?
-#   
+#   # subset index / identify rows to keep in the index:
+#   keep_rownr <- seq_along(names_orig)  # full length row numbers original pseries
+#   names(keep_rownr) <- names_orig
+#   keep_rownr <- keep_rownr[names(result)] # row numbers to keep after subsetting
+#   index <- index[keep_rownr, ]
+# 
+#   # drop unused levels (like in subsetting of pdata.frames)
+#   index <- droplevels(index)
+# 
 #   ### TODO: test for is.null before adding back? see [[.pdata.frame
-#   
-#   attr(result, "index") <- index[names_subsetted, ]
-#   class(result) <- union("pseries", class(x))
+# 
+#   result <- add_pseries_features(result, index)
 #   return(result)
 # }
 
@@ -298,19 +297,13 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     # Nargs_mod to distinguish if called by [] (Nargs_mod == 2L); [,] (Nargs_mod == 3L); [,,] (Nargs_mod == 4L)
     Nargs_mod <- nargs() - (!missing.drop)
   
-    # # Kevin Tappe 2015-10-29 [code from old implementation]
-    # if (missing(drop)){
-    #     if (! missing(j) && length(j) == 1) { drop = TRUE
-    #       } else { drop = FALSE }
-    # }
-  
     old.pdata.frame <- !inherits(x, "data.frame")
     if (! old.pdata.frame){
         # this part for backward compatibility (required by meboot)
       
-        ### subset index appropriately:
+        ### subset index (and row names) appropriately:
         # subsetting data.frame by only j (x[ , j]) or missing j (x[i]) yields full-row
-        # column(s) of data.frame, thus do not subset index because it needs full rows
+        # column(s) of data.frame, thus do not subset the index because it needs full rows (original index)
         #
         # subset index if:
         #      * [i,j] (supplied i AND supplied j) (in this case: Nargs_mod == 3L (or 4L depending on present/missing drop))
@@ -321,19 +314,25 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         #      * [i]   (supplied i AND missing j)                    (Nargs_mod == 2L) [Nargs_mod distinguishes this case from the one where subsetting is needed!]
         #      * [i, drop = TRUE/FALSE] (supplied i AND missing j)   (Nargs_mod == 2L)
         #
-        # => subset index if: supplied i && Nargs_mod => 3L
+        # => subset index (and row names) if: supplied i && Nargs_mod >= 3L
       
-        # Kevin Tappe 2016-01-04 : in case of indexing by a character
-        # vector a pdata.frame, the subseting vector should be converted 
-        # to numeric by matching to the rownames so that the index could 
-        # be correctly indexed (by this numeric value)
-        # (rownames of the pdata.frame and rownames of the pdata.frame's index are not guaranteed to be the same!)
-        ## iindex <- i
         index <- attr(x, "index")
-        if (!missing.i && Nargs_mod >= 3L) { 
+        x.rownames <- row.names(x)
+        if (!missing.i && Nargs_mod >= 3L) {
             iindex <- i
-            if (is.character(iindex)) iindex <- match(iindex, rownames(x))
+            if (is.character(iindex)) {
+              # Kevin Tappe 2016-01-04 : in case of indexing (subsetting) a 
+              # pdata.frame by a character, the subsetting vector should be 
+              # converted to numeric by matching to the row names so that the 
+              # index can be correctly subsetted (by this numeric value).
+              # Motivation:
+              # Row names of the pdata.frame and row names of the pdata.frame's 
+              # index are not guaranteed to be the same!
+              iindex <- match(iindex, rownames(x))
+            }
+            # subset index and row names
             index <- "[.data.frame"(index, iindex, )
+            x.rownames <- x.rownames[iindex]
             
             # remove empty levels in index (if any)
             # NB: really do dropping of unused levels? Standard R behaviour is to leave the levels and not drop unused levels
@@ -344,7 +343,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         }
     }
     
-    # delete attribute for old index first:
+    # delete attribute with old index first:
     # this preseves the order of the attributes because 
     # order of non-standard attributes is scrambled by R's data.frame subsetting with `[.`
     # (need to add new index later anyway)
@@ -354,14 +353,14 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     # (probably by as.data.frame.pdata.frame).
     # Coercing is the built-in behaviour for extraction from data.frames by "[." (see ?`[.data.frame`) 
     # and it seems this cannot be avoided; thus we need to make sure, not to have any coercing going on
-    # which add extra data (such as as.matrix.pseries, as.data.frame.pdata.frame) by setting the class 
+    # which adds extra data (such as as.matrix.pseries, as.data.frame.pdata.frame) by setting the class 
     # to "data.frame" first
     class(x) <- "data.frame"
 
     # call [.data.frame exactly as [.pdata.frame was called but arg is now 'x'
     # this is necessary because there could be several missing arguments
     # use sys.call (and not match.call) because arguments other than drop may not be named
-    # need to evaluate i, j, drop, if supplied, before passing them on (do not pass on as originally catched sys.call)
+    # need to evaluate i, j, drop, if supplied, before passing on (do not pass on as the sys.call caught originally)
     sc_mod <- sc
     sc_mod[[1]] <- quote(`[.data.frame`)
     sc_mod[[2]] <- quote(x)
@@ -369,7 +368,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     if (!missing.i) sc_mod[[3]] <- i # if present, i is always in pos 3
     if (!missing.j) sc_mod[[4]] <- j # if present, j is always in pos 4
     if (!missing.drop) sc_mod[[length(sc)]] <- drop # if present, drop is always in last position (4 or 5,
-                                                    # depending on the call structure an whether missing j or not)
+                                                    # depending on the call structure and whether missing j or not)
     
     mydata <- eval(sc_mod)
 
@@ -381,6 +380,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
       } else {
         # vector or factor -> make it a pseries
         res <- structure(mydata,
+                         names = x.rownames,
                          index = index,
                          class = base::union("pseries", class(mydata)))
       }
@@ -427,6 +427,7 @@ print.pdata.frame <- function(x, ...){
   print(x, ...)
 }
 
+# as.list.pdata.frame:
 # The default is to behave identical to as.list.data.frame.
 # This default is necessary, because some code relies on this 
 # behaviour! Do not change this!
@@ -463,15 +464,24 @@ as.list.pdata.frame <- function(x, keep.attributes = FALSE, ...) {
     return(x)
 }
 
-as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, ...){
+as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, keep.attributes = TRUE, ...){
     index <- attr(x, "index")
-    x <- lapply(x,
-                function(z){
+
+     if (!keep.attributes) {
+       attr(x, "index") <- NULL
+       class(x) <- "data.frame"
+       rownames(x) <- NULL
+     } else {
+      # make each column a pseries
+      x <- lapply(x,
+                  function(z){
+               #     names(z) <- row.names(x) # it does not seem possible to keep the names in the 'pseries' because the call to data.frame later deletes the names 
                     attr(z, "index") <- index
                     class(z) <- base::union("pseries", class(z))
                     return(z)
-                }
-                )
+                  }
+      )
+    }
     
     if (is.null(row.names) || row.names == FALSE) {
         x <- data.frame(x)
@@ -479,9 +489,8 @@ as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, ...
         if (row.names == TRUE) { # set fancy row names
             x <- data.frame(x)
             row.names(x) <- fancy.row.names(index)
-            # using row.names(x)<-"something" is safer (does not allow
-            # duplicate row.names) than
-            # attr(x,"row.names")<-"something"
+            # using row.names(x) <- "something" is safer (does not allow
+            # duplicate row.names) than # attr(x,"row.names") <- "something"
     }
     ## not implemented: if row.names is a character vector, row.names
     ## could also be passed here to base::data.frame, see
@@ -495,7 +504,7 @@ as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, ...
 print.pseries <- function(x, ...){
   attr(x, "index") <- NULL
   attr(x, "class") <- base::setdiff(attr(x, "class"), "pseries")
-  if (length(attr(x, "class")) == 1 && class(x) %in% c("character", "logical", "numeric", "integer")) {
+  if (length(attr(x, "class")) == 1 && class(x) %in% c("character", "logical", "numeric", "integer", "complex")) {
     attr(x, "class") <- NULL
   }
   print(x, ...)
@@ -505,14 +514,14 @@ as.matrix.pseries <- function(x, idbyrow = TRUE, ...){
     index <- attr(x, "index")
     id <- index[[1]]
     time <- index[[2]]
-    time.names <- levels(as.factor(time))
-    x <- split(data.frame(x,time), id)
+    time.names <- levels(time)
+    x <- split(data.frame(x, time), id)
     x <- lapply(x, function(x){
-        rownames(x) <- x[,2]
-        x[,-2,drop=F]
+        rownames(x) <- x[ , 2]
+        x[ , -2, drop = FALSE]
     })
     x <- lapply(x, function(x){
-        x <- x[time.names,,drop=F]
+        x <- x[time.names, , drop = FALSE]
         rownames(x) <- time.names
         x
     }
@@ -537,7 +546,8 @@ plot.pseries <- function(x, plot = c("lattice", "superposed"),
                          scale = FALSE, transparency = TRUE,
                          col = "blue", lwd = 1, ...) {
 
-    if(scale) {scalefun <- function(x) scale(x)
+    if(scale) {
+      scalefun <- function(x) scale(x)
     } else {
         scalefun <- function(x) return(x)}
 
