@@ -6,19 +6,19 @@ pggls <- function(formula, data, subset, na.action,
   # check and match the arguments
     effect <- match.arg(effect)
     model.name <- match.arg(model)
-    if(model.name == "random") {
-        warning("'random' argument to pggls() has been renamed as 'pooling'",
-                call.=FALSE)
+    if (model.name == "random") {
+        warning("for argument 'model' to pggls(), the value 'random' has been renamed as 'pooling'",
+                call. = FALSE)
         model.name <- "pooling"
     }
     data.name <- paste(deparse(substitute(data)))
     cl <- match.call()
     plm.model <- match.call(expand.dots=FALSE)
-    m <- match(c("formula","data","subset","na.action","effect","model","index"),names(plm.model),0)
+    m <- match(c("formula", "data", "subset", "na.action", "effect", "model", "index"), names(plm.model), 0)
     plm.model <- plm.model[c(1,m)]
     plm.model[[1]] <- as.name("plm")
     plm.model$model <- model.name
-    plm.model <- eval(plm.model,parent.frame())
+    plm.model <- eval(plm.model, parent.frame())
     
     index <- attr(model.frame(plm.model), "index")
     pdim <- pdim(plm.model)
@@ -29,7 +29,7 @@ pggls <- function(formula, data, subset, na.action,
     coef.names <- names(coef(plm.model))
     K <- length(coef.names)
     
-    if(model.name=="fd") {
+    if (model.name == "fd") {
     ## eliminate first year in indices
         nt <- pdim$Tint$nt[-1]
         Ti <- pdim$Tint$Ti - 1
@@ -39,9 +39,9 @@ pggls <- function(formula, data, subset, na.action,
         time.names <- pdim$panel.names$time.names[-1]
         tind <- as.numeric(index[,2])
         sel <- (tind-c(-1,tind[-length(tind)]))==1
-        index <- index[sel,]
+        index <- index[sel, ]
         id <- index[[1]]
-        time <- factor(index[[2]], levels=attr(index[,2], "levels")[-1])
+        time <- factor(index[[2]], levels = attr(index[ ,2], "levels")[-1])
     } else {
         nt <- pdim$Tint$nt
         Ti <- pdim$Tint$Ti
@@ -72,7 +72,7 @@ pggls <- function(formula, data, subset, na.action,
         groupsdim <- Ti
     }
     myord <- order(cond, other)
-    X <- model.matrix(plm.model)[myord, , drop=FALSE]
+    X <- model.matrix(plm.model)[myord, , drop = FALSE]
     commonpars <- intersect(names(coef(plm.model)), colnames(X))
     X <- X[, commonpars, drop = FALSE]
     y <- pmodel.response(plm.model)[myord]
@@ -81,16 +81,19 @@ pggls <- function(formula, data, subset, na.action,
     cond <- cond[myord]
     other <- other[myord]
     drop1 <- FALSE
-    if (drop1 && model.name %in% c("within","fd")) {
-    ## drop first time period (see Wooldridge 10.5, eq. 10.61)
-    ## this is needed according to Wooldridge, p.277 but is
-    ## not totally robust to unbalancedness, dummies etc.
+    if (drop1 && model.name %in% c("within", "fd")) {
+    ## drop one time period (e.g. first as we do here)
+    ## (see Wooldridge (2002) 10.5, eq. 10.61)/Woolridge (2010),10.5.5, eq.10.61)
+    ## this is needed according to Wooldridge (2002), p.277 / Wooldridge (2010), p. 312
+    ## but is not totally robust to unbalancedness, dummies etc.
     ## 
-    ## the function turns out to work irrespective of dropping
-    ## one time period or not!! absolutely the same results...
-    ## this is thx to solve.bdsmatrix() using a generalized
+    ## The function turns out to work irrespective of dropping
+    ## one time period or not! Absolutely the same results!
+    ## This is thx to solve.bdsmatrix() using a generalized
     ## inverse, which in this case where rank=T-1 is equivalent
     ## to discarding one year (N columns)
+    ## -> as noted by Wooldridge
+    ##
     ## The 'if' parameterization is just for debugging.
     
         numeric.t <- as.numeric(other)
@@ -112,7 +115,7 @@ pggls <- function(formula, data, subset, na.action,
     if (balanced) {
         for (i in 1:ncond) {
             ut <- resid[cond == lcnd[i]]
-            tres[, , i] <- ut %o% ut
+            tres[ , , i] <- ut %o% ut
         }
         subOmega <- apply(tres, 1:2, mean)
         omega <- bdsmatrix(rep(nother, ncond), rep(subOmega, ncond))
@@ -146,6 +149,7 @@ pggls <- function(formula, data, subset, na.action,
     names(coef) <- rownames(vcov) <- colnames(vcov) <- coef.names
     pmodel <- attr(plm.model, "pmodel")
     pmodel$model.name <- model
+    pmodel$effect.name <- effect
     fullGLS <- list(coefficients  = coef,
                     residuals     = residuals,
                     fitted.values = fitted.values,
@@ -162,13 +166,13 @@ pggls <- function(formula, data, subset, na.action,
 }
 
 summary.pggls <- function(object,...){
-  pmodel <- attr(object,"pmodel")
+  pmodel <- attr(object, "pmodel")
   std.err <- sqrt(diag(object$vcov))
   b <- object$coefficients
   z <- b/std.err
-  p <- 2*pnorm(abs(z),lower.tail=FALSE)
+  p <- 2*pnorm(abs(z), lower.tail = FALSE)
   CoefTable <- cbind(b,std.err,z,p)
-  colnames(CoefTable) <- c("Estimate","Std. Error","z-value","Pr(>|z|)")
+  colnames(CoefTable) <- c("Estimate", "Std. Error", "z-value", "Pr(>|z|)")
   object$CoefTable <- CoefTable
   y <- object$model[[1]]
   object$tss <- tss(y)
@@ -178,14 +182,14 @@ summary.pggls <- function(object,...){
   return(object)
 }
 
-print.summary.pggls <- function(x,digits=max(3, getOption("digits") - 2), width = getOption("width"),...){
-  pmodel <- attr(x,"pmodel")
-  pdim <- attr(x,"pdim")
-  effect <- pmodel$effect
+print.summary.pggls <- function(x, digits = max(3, getOption("digits") - 2), width = getOption("width"), ...){
+  pmodel <- attr(x, "pmodel")
+  pdim <- attr(x, "pdim")
   formula <- pmodel$formula
   model.name <- pmodel$model.name
-  cat(paste(effect.pggls.list[effect]," ",sep=""))
-  cat(paste(model.pggls.list[model.name],"\n",sep=""))
+  effect.name <- pmodel$effect.name
+  cat(paste(effect.pggls.list[effect.name], " ", sep = ""))
+  cat(paste(model.pggls.list[model.name], "\n", sep = ""))
   cat("\nCall:\n")
   print(x$call)
   cat("\n")
@@ -194,9 +198,9 @@ print.summary.pggls <- function(x,digits=max(3, getOption("digits") - 2), width
   print(summary(unlist(residuals(x))))
   cat("\nCoefficients:\n")
   printCoefmat(x$CoefTable,digits=digits)
-  cat(paste("Total Sum of Squares: ",signif(x$tss,digits),"\n",sep=""))
-  cat(paste("Residual Sum of Squares: ",signif(x$ssr,digits),"\n",sep=""))
-  cat(paste("Multiple R-squared: ",signif(x$rsqr,digits),"\n",sep=""))
+  cat(paste("Total Sum of Squares: ",    signif(x$tss,digits),  "\n", sep=""))
+  cat(paste("Residual Sum of Squares: ", signif(x$ssr,digits),  "\n", sep=""))
+  cat(paste("Multiple R-squared: ",      signif(x$rsqr,digits), "\n", sep=""))
   invisible(x)
 }
 
